SystemOrganization addCategory: #'Ma client server tester-Test cases'!

MaNetworkTestCase subclass: #MaClientServerTestCase
	instanceVariableNames: 'server busy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma client server tester-Test cases'!

!MaClientServerTestCase commentStamp: 'cmm 10/13/2005 23:16' prior: 0!
This class tests the functionality of the Ma client server framework.

Because this is a networking program, notice that we subclass from MaNetworkTestCase.  Currently this sort of test case cannot be launched via the SUnit browser.  Instead, you simply type the following and 'do it'.

	MaClientServerTestCase kickoff

While the test methods only appear to test the "byteArray" level, the object level is tested by the testing infrastructure itself!!!

!MaClientServerTestCase class methodsFor: 'player images' stamp: 'cmm 8/26/2005 14:59'!
client1ImageFilename

	^ '_macsTestClient1'! !

!MaClientServerTestCase class methodsFor: 'player images' stamp: 'cmm 8/26/2005 14:59'!
client2ImageFilename

	^ '_macsTestClient2'! !

!MaClientServerTestCase class methodsFor: 'private' stamp: 'cmm 8/24/2005 21:15'!
initialize

	Smalltalk addToStartUpList: self! !

!MaClientServerTestCase class methodsFor: 'kickoff suite' stamp: 'cmm 1/2/2006 21:00'!
kickoffWithoutWarning
	MakoSecurityManager reset.
	self ensureTestEnvironmentDirectory.
	self generateKeyrings.
	super kickoffWithoutWarning.
	self isConductorImage 
		ifTrue: 
			[MakoSecurityManager rememberCurrentWhile: 
					[MakoSecurityManager reset.
					MakoSecurityManager current 
						keyringDirectory: (self keyringDirectoryForPlayerNamed: 'conductor').
					self suite maDebug]]! !

!MaClientServerTestCase class methodsFor: 'accessing' stamp: 'cmm 10/21/2005 16:48'!
playerNames
	^ { 'conductor'.  'client1'.  'client2'.  'server' }! !

!MaClientServerTestCase class methodsFor: 'player images' stamp: 'cmm 8/26/2005 14:59'!
serverImageFilename

	^ '_macsTestServer'! !

!MaClientServerTestCase class methodsFor: 'private' stamp: 'cmm 8/24/2005 21:22'!
setUpSuite
	super setUpSuite.
	self
		addPlayerNamed: 'client1';
		addPlayerNamed: 'client2';
		addPlayerNamed: 'server'! !

!MaClientServerTestCase class methodsFor: 'player images' stamp: 'cmm 12/16/2005 16:34'!
startUp

	super startUp.
	self imageFilename = (self client1ImageFilename, '.image')
		ifTrue: [ self startPlayer: 'client1' on: 1314 ].
	self imageFilename = (self client2ImageFilename, '.image')
		ifTrue: [ self startPlayer: 'client2' on: 1315 ].
	self imageFilename = (self serverImageFilename, '.image')
		ifTrue: [ self startPlayer: 'server' on: 1316 ]! !

!MaClientServerTestCase class methodsFor: 'private' stamp: 'cmm 1/2/2006 21:00'!
tearDownSuite
	super tearDownSuite.
	self playerNames do: 
			[:each | 
			| dir |
			dir := self keyringDirectoryForPlayerNamed: each.
			dir deleteLocalFiles.
			dir containingDirectory deleteDirectory: dir localName]! !

!MaClientServerTestCase class methodsFor: 'private' stamp: 'cmm 8/24/2005 17:01'!
validate

	"Warn if OSProcess is not loaded."
	(Smalltalk hasClassNamed: #OSProcess) ifFalse: [ Warning signal: 'OSProcess is recommended to run the test cases with one-click.  Otherwise you will be prompted to launch several images at various times.' ]! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 10/13/2005 21:02'!
startByteArrayRequests

	| mySocket randomStream randomByteArrayGenerator requestSize largestTest |
	largestTest := 9000.
	randomStream := Random new.
	randomByteArrayGenerator :=
		[ : size | | ba |
		ba := ByteArray new: size.
		1 to: size do: 
			[ :index | ba at: index put: (randomStream nextInt: 256) - 1 ].
		ba ].
	mySocket := MaClientSocket
		hostAddress: (self getAddressFor: 'server')
		port: self class defaultServerPort.
	[ busy := true.
	1 to: 200 do:
		[ : count |  
		| eachRequest eachResponse |
		requestSize := randomStream nextInt: largestTest.
		eachRequest := randomByteArrayGenerator value: requestSize.
		eachResponse := mySocket submit: eachRequest.
		self should: [ eachRequest reverse = eachResponse ] ].
		busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 10/13/2005 20:35'!
startByteArrayServer

	server := MaServerSocket new.
	server
		listenOn: self class defaultServerPort
		answer:
			[ : requestTransmission |
			MacsTransmission newFor: requestTransmission bytes reverse ]! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 8/24/2005 21:41'!
stopByteArrayServer

	server shutdown! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 8/24/2005 20:52'!
testByteArrayLayer

	self
		remotePerform: #startByteArrayServer
		in: 'server' ;

		remotePerform: #startByteArrayRequests
		in: 'client1' ;

		remotePerform: #startByteArrayRequests
		in: 'client2' ;

		remotePerform: #waitUntilDone
		in: 'client1' ;

		remotePerform: #waitUntilDone
		in: 'client2'.

	self 
		remotePerform: #stopByteArrayServer
		in: 'server'! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 8/24/2005 20:50'!
waitUntilDone

	[ busy ] whileTrue: [ (Delay forSeconds: 1) wait ].
	^ nil! !
MaClientServerTestCase initialize!
